/* Sample application to demonstrat FT800 primitives, widgets and customized screen shots */

#include "FT_Platform.h"
#ifdef MSVC_PLATFORM
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <windows.h>
#include <io.h>
#include <direct.h>
#endif

#define SAMAPP_DELAY_BTW_APIS (1000)
#define SAMAPP_ENABLE_DELAY() Ft_Gpu_Hal_Sleep(SAMAPP_DELAY_BTW_APIS)
#define SAMAPP_ENABLE_DELAY_VALUE(x) Ft_Gpu_Hal_Sleep(x)
#define F16(s)        ((ft_int32_t)((s) * 65536))
#define WRITE2CMD(a) Ft_Gpu_Hal_WrCmdBuf(phost,a,sizeof(a))

/* Global variables for display resolution to support various display panels */
/* Default is WQVGA - 480x272 */
ft_int16_t FT_DispWidth = 480;
ft_int16_t FT_DispHeight = 272;
ft_int16_t FT_DispHCycle =  548;
ft_int16_t FT_DispHOffset = 43;
ft_int16_t FT_DispHSync0 = 0;
ft_int16_t FT_DispHSync1 = 41;
ft_int16_t FT_DispVCycle = 292;
ft_int16_t FT_DispVOffset = 12;
ft_int16_t FT_DispVSync0 = 0;
ft_int16_t FT_DispVSync1 = 10;
ft_uint8_t FT_DispPCLK = 5;
ft_char8_t FT_DispSwizzle = 0;
ft_char8_t FT_DispPCLKPol = 1;

/* Global used for buffer optimization */
Ft_Gpu_Hal_Context_t host,*phost;


ft_uint32_t Ft_CmdBuffer_Index;
ft_uint32_t Ft_DlBuffer_Index;

#ifdef BUFFER_OPTIMIZATION
ft_uint8_t  Ft_DlBuffer[FT_DL_SIZE];
ft_uint8_t  Ft_CmdBuffer[FT_CMD_FIFO_SIZE];
#endif

ft_void_t Ft_App_WrCoCmd_Buffer(Ft_Gpu_Hal_Context_t *phost,ft_uint32_t cmd)
{
#ifdef  BUFFER_OPTIMIZATION
   /* Copy the command instruction into buffer */
   ft_uint32_t *pBuffcmd;
   pBuffcmd =(ft_uint32_t*)&Ft_CmdBuffer[Ft_CmdBuffer_Index];
   *pBuffcmd = cmd;
#endif
#ifdef ARDUINO_PLATFORM
   Ft_Gpu_Hal_WrCmd32(phost,cmd);
#endif

   /* Increment the command index */
   Ft_CmdBuffer_Index += FT_CMD_SIZE;  
}

ft_void_t Ft_App_WrDlCmd_Buffer(Ft_Gpu_Hal_Context_t *phost,ft_uint32_t cmd)
{
#ifdef BUFFER_OPTIMIZATION  
   /* Copy the command instruction into buffer */
   ft_uint32_t *pBuffcmd;
   pBuffcmd =(ft_uint32_t*)&Ft_DlBuffer[Ft_DlBuffer_Index];
   *pBuffcmd = cmd;
#endif

#ifdef ARDUINO_PLATFORM
   Ft_Gpu_Hal_Wr32(phost,(RAM_DL+Ft_DlBuffer_Index),cmd);
#endif
   /* Increment the command index */
   Ft_DlBuffer_Index += FT_CMD_SIZE;  
}

ft_void_t Ft_App_WrCoStr_Buffer(Ft_Gpu_Hal_Context_t *phost,const ft_char8_t *s)
{
#ifdef  BUFFER_OPTIMIZATION  
  ft_uint16_t length = 0;
  length = strlen(s) + 1;//last for the null termination
  
  strcpy(&Ft_CmdBuffer[Ft_CmdBuffer_Index],s);  

  /* increment the length and align it by 4 bytes */
  Ft_CmdBuffer_Index += ((length + 3) & ~3);  
#endif  
}

ft_void_t Ft_App_Flush_DL_Buffer(Ft_Gpu_Hal_Context_t *phost)
{
#ifdef  BUFFER_OPTIMIZATION    
   if (Ft_DlBuffer_Index> 0)
     Ft_Gpu_Hal_WrMem(phost,RAM_DL,Ft_DlBuffer,Ft_DlBuffer_Index);
#endif     
   Ft_DlBuffer_Index = 0;
   
}

ft_void_t Ft_App_Flush_Co_Buffer(Ft_Gpu_Hal_Context_t *phost)
{
#ifdef  BUFFER_OPTIMIZATION    
   if (Ft_CmdBuffer_Index > 0)
     Ft_Gpu_Hal_WrCmdBuf(phost,Ft_CmdBuffer,Ft_CmdBuffer_Index);
#endif     
   Ft_CmdBuffer_Index = 0;
}


/* API to give fadeout effect by changing the display PWM from 100 till 0 */
ft_void_t SAMAPP_fadeout()
{
   ft_int32_t i;
	
	for (i = 100; i >= 0; i -= 3) 
	{
		Ft_Gpu_Hal_Wr8(phost,REG_PWM_DUTY,i);

		Ft_Gpu_Hal_Sleep(2);//sleep for 2 ms
	}
}

/* API to perform display fadein effect by changing the display PWM from 0 till 100 and finally 128 */
ft_void_t SAMAPP_fadein()
{
	ft_int32_t i;
	
	for (i = 0; i <=100 ; i += 3) 
	{
		Ft_Gpu_Hal_Wr8(phost,REG_PWM_DUTY,i);
		Ft_Gpu_Hal_Sleep(2);//sleep for 2 ms
	}
	/* Finally make the PWM 100% */
	i = 128;
	Ft_Gpu_Hal_Wr8(phost,REG_PWM_DUTY,i);
}


/* API to check the status of previous DLSWAP and perform DLSWAP of new DL */
/* Check for the status of previous DLSWAP and if still not done wait for few ms and check again */
ft_void_t SAMAPP_GPU_DLSwap(ft_uint8_t DL_Swap_Type)
{
	ft_uint8_t Swap_Type = DLSWAP_FRAME,Swap_Done = DLSWAP_FRAME;

	if(DL_Swap_Type == DLSWAP_LINE)
	{
		Swap_Type = DLSWAP_LINE;
	}

	/* Perform a new DL swap */
	Ft_Gpu_Hal_Wr8(phost,REG_DLSWAP,Swap_Type);

	/* Wait till the swap is done */
	while(Swap_Done)
	{
		Swap_Done = Ft_Gpu_Hal_Rd8(phost,REG_DLSWAP);

		if(DLSWAP_DONE != Swap_Done)
		{
			Ft_Gpu_Hal_Sleep(10);//wait for 10ms
		}
	}	
}


ft_void_t Ft_BootupConfig()
{

	/* Do a power cycle for safer side */
	Ft_Gpu_Hal_Powercycle(phost,FT_TRUE);
	Ft_Gpu_Hal_Rd16(phost,RAM_G);
	/* Set the clk to external clock */
	Ft_Gpu_HostCommand(phost,FT_GPU_EXTERNAL_OSC);  
	Ft_Gpu_Hal_Sleep(10);
	  

	/* Switch PLL output to 48MHz */
	Ft_Gpu_HostCommand(phost,FT_GPU_PLL_48M);  
	Ft_Gpu_Hal_Sleep(10);

	/* Do a core reset for safer side */
	Ft_Gpu_HostCommand(phost,FT_GPU_CORE_RESET);     

	/* Access address 0 to wake up the FT800 */
	Ft_Gpu_HostCommand(phost,FT_GPU_ACTIVE_M);  

    Ft_Gpu_Hal_Wr8(phost, REG_GPIO_DIR,0x80 | Ft_Gpu_Hal_Rd8(phost,REG_GPIO_DIR));
    Ft_Gpu_Hal_Wr8(phost, REG_GPIO,0x080 | Ft_Gpu_Hal_Rd8(phost,REG_GPIO));
	
	{
		ft_uint8_t chipid;
		//Read Register ID to check if FT800 is ready. 
		chipid = Ft_Gpu_Hal_Rd8(phost, REG_ID);
		while(chipid != 0x7C)
			chipid = Ft_Gpu_Hal_Rd8(phost, REG_ID);
#ifdef MSVC_PLATFORM
		printf("VC1 register ID after wake up %x\n",chipid);
#endif
	}
	/* Configuration of LCD display */
#ifdef SAMAPP_DISPLAY_QVGA	
	/* Values specific to QVGA LCD display */
	FT_DispWidth = 320;
	FT_DispHeight = 240;
	FT_DispHCycle =  408;
	FT_DispHOffset = 70;
	FT_DispHSync0 = 0;
	FT_DispHSync1 = 10;
	FT_DispVCycle = 263;
	FT_DispVOffset = 13;
	FT_DispVSync0 = 0;
	FT_DispVSync1 = 2;
	FT_DispPCLK = 8;
	FT_DispSwizzle = 2;
	FT_DispPCLKPol = 0;
#endif

    Ft_Gpu_Hal_Wr16(phost, REG_HCYCLE, FT_DispHCycle);
    Ft_Gpu_Hal_Wr16(phost, REG_HOFFSET, FT_DispHOffset);
    Ft_Gpu_Hal_Wr16(phost, REG_HSYNC0, FT_DispHSync0);
    Ft_Gpu_Hal_Wr16(phost, REG_HSYNC1, FT_DispHSync1);
    Ft_Gpu_Hal_Wr16(phost, REG_VCYCLE, FT_DispVCycle);
    Ft_Gpu_Hal_Wr16(phost, REG_VOFFSET, FT_DispVOffset);
    Ft_Gpu_Hal_Wr16(phost, REG_VSYNC0, FT_DispVSync0);
    Ft_Gpu_Hal_Wr16(phost, REG_VSYNC1, FT_DispVSync1);
    Ft_Gpu_Hal_Wr8(phost, REG_SWIZZLE, FT_DispSwizzle);
    Ft_Gpu_Hal_Wr8(phost, REG_PCLK_POL, FT_DispPCLKPol);
    Ft_Gpu_Hal_Wr8(phost, REG_PCLK,FT_DispPCLK);//after this display is visible on the LCD
    Ft_Gpu_Hal_Wr16(phost, REG_HSIZE, FT_DispWidth);
    Ft_Gpu_Hal_Wr16(phost, REG_VSIZE, FT_DispHeight);


    /* Touch configuration - configure the resistance value to 1200 - this value is specific to customer requirement and derived by experiment */
    Ft_Gpu_Hal_Wr16(phost, REG_TOUCH_RZTHRESH,1200);
    Ft_Gpu_Hal_Wr8(phost, REG_GPIO_DIR,0xff);
        Ft_Gpu_Hal_Wr8(phost, REG_GPIO,0x0ff);

}

#define SAMAPP_ENABLE_APIS_SET0
#ifdef SAMAPP_ENABLE_APIS_SET0

/* Optimized implementation of sin and cos table - precision is 16 bit */
FT_PROGMEM ft_prog_uint16_t sintab[] = {
  0, 402, 804, 1206, 1607, 2009, 2410, 2811, 3211, 3611, 4011, 4409, 4807, 5205, 5601, 5997, 6392, 
  6786, 7179, 7571, 7961, 8351, 8739, 9126, 9511, 9895, 10278, 10659, 11038, 11416, 11792, 12166, 12539,
  12909, 13278, 13645, 14009, 14372, 14732, 15090, 15446, 15799, 16150, 16499, 16845, 17189, 17530, 17868,
  18204, 18537, 18867, 19194, 19519, 19840, 20159, 20474, 20787, 21096, 21402, 21705, 22004, 22301, 22594, 
  22883, 23169, 23452, 23731, 24006, 24278, 24546, 24811, 25072, 25329, 25582, 25831, 26077, 26318, 26556, 26789, 
  27019, 27244, 27466, 27683, 27896, 28105, 28309, 28510, 28706, 28897, 29085, 29268, 29446, 29621, 29790, 29955, 
  30116, 30272, 30424, 30571, 30713, 30851, 30984, 31113, 31236, 31356, 31470, 31580, 31684, 31785, 31880, 31970, 
  32056, 32137, 32213, 32284, 32350, 32412, 32468, 32520, 32567, 32609, 32646, 32678, 32705, 32727, 32744, 32757, 
  32764, 32767, 32764};


 static unsigned char digits[] = {120,156,237,93,93,108,92,199,117,190,251,203,229,82,178,72,35,78,90,147,20,55,70,144,22,182,73,201,113,145,6,104,205,101,27,163,15,65,33,18,5,140,162,6,234,101
,81,52,232,31,74,182,125,41,208,135,221,182,121,224,67,81,170,47,65,141,194,224,62,245,173,89,59,40,242,226,162,43,6,125,42,154,48,82,243,80,4,112,73,91,78,138
,20,133,54,78,101,57,38,69,78,231,204,220,159,115,230,206,119,21,81,148,66,89,243,1,178,69,125,156,185,247,206,156,123,230,252,205,220,86,171,24,79,60,121,254,179,11
,237,207,76,181,102,231,159,158,124,122,242,194,66,138,143,125,236,147,79,183,22,62,151,252,248,100,198,196,191,212,122,246,252,211,207,124,2,243,63,51,243,169,79,207,151,163
,40,154,214,127,62,171,255,252,186,254,115,243,59,71,42,10,8,8,8,8,8,8,8,8,8,120,164,208,135,76,115,15,82,131,111,33,102,66,189,142,168,161,66,215,154,86
,234,178,159,41,13,149,234,193,70,106,21,220,158,166,86,252,212,142,166,150,252,212,174,166,90,126,106,164,169,73,255,13,106,6,120,17,101,205,28,250,169,138,166,110,251,169
,154,166,14,252,84,93,83,31,250,169,49,77,253,200,79,141,107,234,3,63,213,212,212,251,126,106,66,83,55,113,171,255,243,83,37,61,134,63,244,83,209,130,82,72,106,170
,234,176,5,168,104,237,26,98,162,49,216,232,228,241,140,127,250,53,42,35,40,241,139,104,186,116,35,40,107,139,88,214,70,199,145,181,234,195,44,107,103,177,172,61,166,169
,247,252,212,57,77,253,192,79,93,208,212,30,166,128,132,210,64,161,89,238,42,168,215,154,80,175,145,54,92,71,212,44,210,107,122,158,71,64,175,105,180,47,66,170,214,130
,212,137,226,12,22,40,154,201,125,63,245,24,22,168,115,88,160,104,186,110,249,169,54,150,181,101,44,80,29,133,149,87,192,35,130,18,208,243,36,82,87,140,97,145,19,172
,101,163,28,118,124,74,108,77,145,114,216,245,201,156,86,27,218,178,25,249,148,216,166,254,199,117,99,88,228,148,24,221,192,42,41,251,188,18,27,146,101,67,134,133,190,79
,7,116,111,23,73,135,229,149,24,221,64,139,148,125,94,137,25,203,134,148,125,206,2,163,123,59,50,202,62,175,196,118,232,93,166,107,29,181,92,170,109,46,162,159,238,186
,203,232,203,208,18,213,84,71,30,29,54,188,74,151,220,205,55,138,162,121,243,235,11,75,30,170,196,254,27,16,16,16,16,16,16,240,168,161,217,179,255,127,33,103,91,148
,134,214,183,168,142,222,113,169,153,56,130,244,138,58,116,44,194,210,142,53,244,200,104,218,118,174,20,123,225,30,215,228,76,236,133,47,231,45,200,196,51,234,228,45,200,115
,177,23,222,205,187,38,115,177,211,180,153,183,32,219,241,229,183,242,22,100,114,249,97,222,215,165,86,239,196,173,28,175,85,155,115,135,52,120,19,50,38,80,249,175,27,223
,211,38,238,181,210,191,220,184,170,71,165,87,189,113,227,221,152,162,32,198,74,227,246,164,30,70,253,159,15,232,217,98,211,174,58,50,143,249,91,101,178,138,95,47,173,24
,219,211,14,63,13,0,253,218,11,202,142,8,61,154,29,126,19,249,32,59,116,96,126,165,101,44,224,56,252,71,151,162,219,154,161,255,95,143,123,177,163,95,53,141,198,182
,41,44,117,216,154,94,55,215,138,29,232,14,253,210,230,193,228,188,210,15,176,115,139,134,56,185,249,218,198,198,138,54,110,95,47,111,108,16,189,82,223,216,224,162,51,176
,211,69,33,28,103,186,140,137,125,49,155,108,6,99,71,183,172,175,235,184,193,13,162,38,179,201,102,104,26,67,219,10,148,227,6,143,199,174,246,121,143,223,49,180,214,60
,13,53,15,23,116,245,200,205,232,75,92,210,127,150,245,211,14,244,220,100,157,93,45,237,246,232,1,122,181,163,139,211,153,171,64,35,126,123,242,211,198,149,185,21,253,26
,73,127,252,0,211,202,122,20,230,254,245,133,230,205,255,47,167,51,169,39,99,65,89,207,194,204,164,186,154,62,173,158,228,106,60,201,77,51,201,45,211,35,137,222,181,210
,215,76,243,94,229,154,145,210,237,116,36,110,235,155,95,173,211,64,92,58,36,193,58,72,94,204,63,219,248,227,242,142,22,235,223,221,32,105,218,46,109,108,112,175,132,110
,107,201,60,125,250,184,108,156,200,89,170,164,178,148,97,215,142,128,185,203,37,73,141,204,35,153,247,95,57,42,37,30,128,100,178,25,140,196,174,199,143,47,27,37,151,79
,38,219,165,150,44,229,198,240,54,237,155,76,178,237,168,33,61,157,71,75,246,209,15,114,238,236,192,250,81,229,81,223,101,162,241,248,150,63,151,99,2,30,29,192,64,119
,52,13,3,221,165,29,16,218,139,162,103,97,12,148,86,128,130,56,242,145,159,74,214,88,15,38,212,9,71,31,231,242,106,55,193,51,111,190,249,230,63,249,169,227,96,92
,165,112,67,32,131,140,82,210,76,105,50,198,49,83,120,35,169,31,198,4,195,205,148,178,137,19,101,96,211,246,149,214,44,103,216,112,53,213,118,101,196,168,94,70,13,245
,100,44,123,27,77,40,40,20,109,229,137,95,89,216,56,155,23,54,206,230,69,87,193,12,226,166,242,196,175,44,6,249,21,33,193,80,21,38,23,65,16,126,55,183,34,164
,160,193,193,201,69,240,26,148,177,64,87,240,56,85,224,11,66,163,209,47,165,2,32,77,212,230,190,181,174,60,242,89,234,149,152,212,56,102,116,214,200,149,79,41,160,61
,65,45,48,198,185,219,10,163,250,213,190,224,58,41,179,31,45,75,59,42,147,249,203,85,56,229,212,1,72,26,145,89,180,231,167,10,146,70,5,242,218,41,150,87,240,110
,108,98,81,222,194,242,122,252,100,56,16,101,156,12,183,33,99,47,202,88,148,233,230,223,205,126,154,206,166,248,168,53,193,197,141,138,1,82,104,143,227,186,183,17,153,121
,211,173,172,145,84,106,124,66,198,5,35,100,222,120,39,64,64,23,57,35,87,142,42,167,28,57,91,203,24,176,166,32,148,95,212,240,83,13,60,172,119,88,40,65,74,248
,140,130,122,133,214,80,144,18,62,151,27,137,20,143,41,152,139,171,98,161,213,111,8,104,164,13,111,107,220,122,241,219,144,9,56,45,120,226,53,196,252,33,211,30,218,249
,102,90,171,204,21,203,162,48,91,206,48,202,4,23,178,102,93,70,145,202,205,212,76,153,105,170,90,172,107,88,127,9,85,54,166,77,42,244,221,140,170,91,53,208,171,77
,102,253,197,139,213,160,71,130,125,43,90,123,61,235,207,46,140,211,244,175,250,6,199,226,87,204,244,183,74,127,35,213,182,82,154,154,34,29,79,218,191,146,245,71,237,41
,123,216,136,21,7,235,239,66,252,110,37,111,102,214,159,89,124,40,44,16,191,153,172,191,52,207,61,103,59,100,253,153,14,222,203,58,102,253,153,197,231,7,105,199,188,63
,179,194,124,43,237,152,247,103,168,126,76,189,45,250,35,227,96,127,210,62,215,97,139,143,159,209,217,151,43,223,120,235,63,244,202,176,109,43,172,50,195,98,70,187,255,52
,91,227,212,120,36,52,72,105,169,70,255,112,45,34,9,24,58,198,131,81,191,214,8,217,148,134,74,153,169,235,53,71,35,117,50,29,223,118,244,226,88,166,227,39,190,228
,172,135,91,176,52,33,26,127,243,77,92,194,17,227,147,136,120,226,53,208,109,249,175,99,49,45,189,244,82,75,82,105,77,198,124,206,62,235,198,171,7,173,118,210,62,51
,243,75,207,107,164,92,196,23,204,252,190,29,191,107,114,140,205,252,174,36,111,20,31,174,100,126,43,118,30,46,187,253,233,27,219,90,201,21,22,197,242,210,212,2,167,231
,108,181,206,250,27,153,254,244,107,115,116,177,250,226,231,163,205,92,127,51,177,212,140,171,92,127,217,187,225,246,103,238,142,94,182,11,25,53,17,223,95,55,126,109,218,25
,181,22,203,115,18,131,93,86,110,127,230,5,216,139,72,144,220,254,210,24,108,87,185,253,165,49,216,142,114,251,163,101,253,131,104,106,106,178,174,220,254,232,98,43,53,122
,113,215,220,254,244,111,93,167,159,142,46,142,89,166,148,246,167,209,106,240,87,172,44,222,55,99,167,167,213,121,252,125,179,186,49,93,139,119,184,44,24,169,73,149,237,22
,23,60,18,155,204,166,147,239,219,50,189,188,217,15,92,182,170,95,252,98,43,253,97,206,21,228,12,103,113,237,212,4,116,125,162,6,236,47,170,193,254,66,113,65,192,169
,71,234,254,228,37,124,34,102,114,25,46,171,127,44,220,208,114,35,101,114,145,132,11,25,229,56,156,45,230,105,73,223,167,118,139,226,215,49,164,195,180,166,150,218,254,70
,117,253,52,169,75,191,234,52,242,60,141,5,93,231,1,121,180,180,122,114,215,235,229,172,243,170,28,184,58,27,96,74,40,176,53,179,203,210,21,244,180,153,235,69,163,153
,234,201,134,24,158,14,239,131,126,47,179,23,104,253,74,87,87,154,251,204,193,218,226,63,37,246,102,70,165,153,199,179,98,102,103,21,139,138,60,38,188,60,50,40,250,201
,15,53,99,241,38,232,45,234,231,106,38,189,108,222,180,139,178,193,173,170,122,163,52,76,146,89,205,37,38,107,106,233,101,189,162,103,235,198,38,163,174,27,33,79,228,91
,184,224,7,50,38,204,131,78,215,76,2,47,30,253,103,38,89,168,234,112,114,186,181,152,100,78,202,163,215,153,132,110,151,118,174,214,146,39,91,212,35,157,198,2,40,213
,121,16,253,94,124,161,29,17,50,160,14,210,97,27,9,211,48,73,78,25,72,89,227,193,68,71,214,76,209,94,252,119,71,214,120,31,82,214,68,48,177,38,100,77,84,143
,75,89,19,213,227,82,214,68,31,82,214,68,245,184,148,53,97,228,73,89,19,213,227,103,185,224,201,234,113,89,249,45,170,199,199,132,132,210,205,247,210,159,182,212,251,229
,36,84,189,173,53,133,8,72,175,164,1,238,125,221,9,211,68,165,127,101,17,164,94,244,229,136,35,85,161,249,152,14,11,247,184,129,95,22,75,119,131,28,89,128,51,31
,228,232,38,212,187,46,147,222,135,175,124,50,32,32,32,5,42,240,38,167,240,138,159,234,168,227,36,158,238,16,173,63,94,226,105,201,79,237,170,66,219,166,229,167,228,122
,195,81,144,120,178,5,222,126,180,241,86,152,49,20,112,142,226,2,111,63,230,177,66,11,222,87,64,64,64,64,64,192,253,195,212,212,83,79,181,124,132,245,193,125,118,76
,211,235,219,16,108,10,223,155,192,125,214,52,242,237,242,178,85,68,222,45,226,54,116,237,6,169,8,181,239,146,165,178,239,179,246,214,142,90,11,254,34,11,221,219,213,218
,255,126,217,195,144,205,6,242,226,119,8,4,1,187,199,9,4,113,56,129,32,14,233,156,11,72,231,92,64,58,231,2,19,96,162,34,235,156,23,108,203,6,91,101,11,182
,101,207,97,19,251,66,190,56,151,81,190,249,37,212,97,35,61,95,168,145,30,41,92,226,120,226,248,13,200,212,145,119,64,94,5,202,30,150,177,15,64,178,6,210,24,119
,216,231,95,32,107,192,210,47,144,181,230,177,101,13,108,203,46,144,181,115,197,178,86,112,4,0,40,217,106,99,9,109,226,232,74,105,71,93,211,138,205,235,197,204,30,182
,102,192,224,151,223,48,145,165,190,183,79,19,143,244,143,190,81,161,126,39,209,150,206,121,95,218,74,65,128,168,99,56,175,240,216,192,45,138,169,7,4,4,20,195,26,67
,94,13,16,71,150,189,229,158,54,72,236,213,0,113,20,123,213,199,181,13,245,182,143,170,89,5,208,242,113,20,91,190,222,240,42,103,109,60,28,61,55,240,151,144,14,212
,245,25,104,36,84,118,161,202,108,32,203,49,46,202,92,245,82,38,202,237,47,158,49,69,31,75,94,138,214,15,16,1,163,245,3,36,254,230,112,112,140,214,15,80,88,185
,140,237,189,14,94,131,215,240,26,188,140,215,224,58,94,131,105,58,193,164,232,161,63,130,245,71,91,94,235,219,54,107,65,234,196,241,11,240,90,213,17,12,21,174,65,91
,112,28,202,154,241,70,86,189,84,243,164,101,237,177,99,203,26,240,45,218,88,214,150,143,37,107,93,44,107,155,88,214,182,176,189,55,15,172,155,234,223,127,71,107,62,239
,165,244,91,178,62,227,157,45,18,247,155,165,175,161,187,203,159,54,66,176,117,7,222,216,253,0,154,67,182,150,220,119,236,152,213,215,94,71,212,110,126,244,250,28,63,247
,249,231,159,154,10,6,84,64,128,11,252,86,148,255,19,82,243,112,7,79,101,4,29,231,75,80,211,155,66,139,43,126,106,23,105,15,155,87,243,239,211,49,5,203,189,220
,63,87,39,179,93,192,165,37,70,148,127,127,244,221,116,23,112,105,243,232,31,90,226,250,95,141,55,33,151,254,70,104,245,93,115,21,170,227,60,136,136,97,97,33,163,218
,62,48,101,206,182,132,38,243,172,77,204,238,104,242,188,190,11,107,41,102,142,176,117,2,215,199,245,162,98,205,210,236,17,108,101,215,219,101,245,67,179,233,153,47,113,182
,242,224,131,104,176,110,125,207,239,177,7,251,35,123,177,95,181,55,244,223,124,178,75,91,230,98,229,90,142,209,3,69,143,246,118,84,125,73,227,174,213,102,227,213,87,95
,253,59,167,213,39,70,42,221,167,254,109,193,212,71,230,14,191,100,207,23,248,91,206,152,241,189,89,221,143,79,33,96,11,69,60,58,179,170,117,222,252,45,11,78,216,218
,163,219,250,121,47,91,191,57,51,168,237,74,244,142,30,191,247,236,197,178,229,207,88,6,106,181,77,255,70,107,48,155,228,142,253,205,29,186,73,187,113,62,165,104,76,63
,180,117,243,171,209,47,138,73,249,248,87,212,126,203,10,212,58,45,235,255,195,119,197,252,252,197,248,62,87,181,228,253,105,110,252,204,132,44,121,135,214,60,82,129,61,223
,242,82,230,193,189,76,209,230,37,10,114,228,203,92,44,102,113,153,75,101,132,45,216,223,244,223,31,33,164,146,3,2,2,2,62,10,192,54,67,233,223,32,53,15,179,49
,5,70,229,43,10,110,173,46,176,28,213,221,90,142,132,120,161,247,193,172,230,254,219,184,211,106,222,130,29,162,189,50,29,159,75,111,252,226,245,186,54,65,103,148,19,40
,152,177,102,208,102,124,76,192,138,211,72,223,93,53,62,38,128,5,172,172,51,173,174,39,219,176,217,122,63,180,212,65,90,242,154,13,114,92,139,109,11,187,149,240,211,99
,195,108,242,241,248,64,10,238,167,155,66,221,237,210,55,237,41,168,34,136,68,6,10,157,16,117,213,24,129,111,228,158,91,95,8,5,157,46,225,249,223,82,48,53,149,236
,22,243,224,248,94,74,223,79,29,75,12,233,14,209,89,237,77,156,168,141,6,56,81,59,145,31,215,128,128,128,128,128,128,128,128,128,211,3,20,43,59,222,137,105,187,48
,164,120,188,141,43,119,177,41,151,225,196,235,227,10,190,131,83,83,254,220,34,161,227,57,175,51,70,21,127,73,38,250,21,200,4,4,4,4,60,188,224,27,253,165,74,229
,167,3,56,107,207,150,164,88,100,201,57,242,143,71,150,214,28,138,133,84,38,28,134,69,64,216,225,0,185,200,210,130,96,68,4,164,2,27,5,4,4,4,4,4,4,4
,4,4,60,84,232,42,184,41,47,57,67,214,3,50,217,193,166,188,161,130,153,207,29,5,51,159,100,151,247,253,20,69,150,122,94,198,100,62,253,213,165,5,9,248,128,128
,135,18,48,39,95,125,13,80,165,151,71,96,7,239,199,135,232,125,251,3,242,179,189,253,153,178,12,127,127,102,251,155,191,191,173,208,223,67,210,95,132,251,139,240,214,103
,74,37,128,254,104,113,65,223,5,88,198,135,79,205,193,254,162,179,135,127,9,152,168,118,178,135,233,85,242,197,214,49,241,231,35,144,43,160,179,146,65,242,129,230,3,236
,254,162,249,240,39,58,78,187,188,132,254,36,229,239,175,132,223,55,178,122,208,251,49,192,239,71,23,191,31,109,252,29,142,57,216,95,116,6,246,23,141,195,254,162,58,236
,47,42,135,253,115,1,143,32,100,158,128,39,17,106,173,142,164,182,203,9,213,184,50,230,52,250,66,66,141,127,40,147,35,219,213,239,39,84,83,173,243,60,199,193,228,34
,59,166,247,38,255,140,210,27,149,17,59,166,215,28,166,148,52,138,22,197,49,189,87,179,111,195,245,203,163,44,17,51,71,235,90,114,255,251,148,40,73,183,168,209,94,179
,94,250,252,84,0,156,238,175,106,43,238,8,210,94,235,212,153,164,190,50,71,144,250,216,75,126,144,71,116,82,31,169,159,217,21,215,90,230,125,208,40,101,142,96,135,247
,33,43,29,168,143,52,53,63,84,188,220,122,147,247,161,31,132,85,58,108,241,62,100,165,3,245,145,86,75,208,248,164,140,248,94,151,115,68,231,174,146,71,66,239,203,86
,137,216,84,229,170,178,160,196,105,209,220,214,168,114,251,98,83,170,250,53,150,177,154,144,6,74,131,165,185,74,61,65,69,127,17,221,63,212,229,181,26,204,137,239,138,242
,253,210,32,51,182,198,100,113,252,179,108,78,218,226,40,50,58,202,130,203,26,75,0,190,226,202,90,42,12,38,98,224,151,53,179,186,95,73,126,18,178,102,74,246,251,201
,79,66,214,204,91,237,151,53,147,12,77,187,23,178,38,15,212,16,178,38,15,57,16,178,38,14,57,144,85,53,230,144,131,164,123,89,85,35,14,57,144,219,8,230,120,247
,244,144,217,126,85,113,200,1,61,100,86,17,78,55,207,143,31,63,72,15,106,63,106,13,88,247,227,230,108,246,68,215,92,157,96,178,60,161,245,223,124,170,161,110,79,237
,100,87,62,163,149,230,110,74,169,254,44,59,126,252,144,53,82,234,71,229,221,84,138,206,93,147,95,135,91,93,72,85,222,108,139,55,82,234,102,37,21,162,199,197,103,50
,41,243,252,203,233,83,138,47,202,41,92,31,20,16,16,112,108,252,52,166,254,25,50,19,192,1,215,24,128,147,24,73,177,162,35,178,181,178,71,135,246,52,176,239,182,9
,207,243,33,117,9,14,206,35,101,92,112,112,30,8,196,147,158,222,243,83,109,85,248,173,211,43,126,170,163,160,227,41,150,75,137,227,159,158,14,42,77,185,237,228,96,168
,96,17,234,172,130,95,134,209,246,44,40,39,165,225,128,42,188,142,182,79,105,252,9,100,78,28,99,184,156,116,11,138,245,52,156,147,242,14,148,53,90,220,64,62,104,4
,197,208,216,78,123,94,202,184,68,126,49,52,102,149,63,31,100,54,247,245,189,148,52,171,4,164,89,37,240,32,207,41,43,56,59,202,60,151,151,49,239,16,58,94,161,9
,163,135,246,187,221,0,51,40,28,73,175,3,20,155,104,17,127,240,182,138,246,214,106,252,14,166,2,2,2,238,5,56,178,196,63,165,110,144,69,150,248,167,212,109,163,47
,176,46,183,68,163,202,247,25,133,34,75,145,252,64,55,143,44,17,50,111,234,32,90,144,219,240,65,100,201,96,57,102,246,169,3,96,29,137,200,146,132,136,44,73,124,84
,173,163,33,182,142,118,177,117,180,3,87,44,17,89,114,80,113,210,83,153,0,188,175,135,67,24,78,217,87,217,142,46,142,9,195,137,75,205,59,50,178,196,62,229,230,30
,92,204,63,0,231,12,32,47,214,22,235,95,229,171,252,85,233,115,234,210,225,100,39,101,196,202,168,151,229,171,217,171,34,102,177,141,143,58,167,206,64,45,56,9,3,240
,41,186,152,34,97,0,107,239,64,65,119,131,132,1,184,27,36,12,224,133,162,247,17,188,80,10,190,80,39,110,231,213,114,227,157,130,230,3,236,175,34,59,175,159,69,150
,56,165,237,188,15,89,100,137,83,90,169,173,179,200,18,239,250,140,186,37,34,75,140,58,171,86,68,100,137,81,231,222,117,34,75,25,117,254,162,19,89,202,168,124,100,41
,227,66,100,41,32,224,158,241,208,25,75,139,82,183,159,132,177,4,124,248,251,98,44,129,154,78,26,234,190,159,234,170,66,99,9,88,68,52,9,171,126,106,168,10,67,73
,192,34,210,227,132,44,34,158,134,115,176,128,29,245,170,227,168,115,99,105,77,26,11,220,88,106,136,213,232,193,25,75,242,85,233,143,203,113,239,164,204,126,105,40,29,127
,102,44,61,11,231,132,196,24,200,154,72,195,9,200,52,156,192,253,137,23,249,101,205,228,218,86,189,212,233,63,215,94,164,225,12,252,105,56,130,76,195,137,160,144,76,195
,113,189,134,210,112,162,145,76,195,201,70,148,134,203,159,32,22,131,165,225,162,156,177,244,75,41,21,140,165,128,128,251,136,92,169,109,41,85,163,255,232,170,171,153,126,242
,183,97,95,50,165,157,180,213,142,227,247,78,103,202,119,228,104,212,97,166,124,157,175,204,53,51,229,91,114,22,193,129,172,129,96,75,197,184,146,245,54,236,70,182,148,172
,129,200,20,103,137,43,95,90,97,50,197,41,74,44,228,87,222,196,71,228,26,226,90,162,196,66,30,58,34,14,46,145,31,229,18,221,203,175,188,137,15,123,209,10,35,191
,206,158,246,33,115,139,226,195,94,242,163,92,162,123,90,124,246,82,74,124,216,139,22,159,190,104,149,246,177,44,22,152,50,183,1,218,114,249,94,102,97,182,134,110,148,25
,55,43,117,62,177,131,62,179,136,174,71,155,204,140,250,41,110,17,29,181,26,200,140,114,190,246,213,228,235,134,20,202,50,95,112,28,99,163,157,49,251,209,134,160,106,25
,213,127,210,177,41,186,9,67,145,9,148,91,121,65,33,27,128,202,202,79,244,123,35,29,117,194,223,27,217,82,133,113,43,144,51,26,170,194,221,6,199,139,91,249,221,30
,243,38,3,83,148,158,11,248,54,244,14,93,241,83,116,247,192,237,33,145,236,3,74,219,64,61,64,105,123,6,184,61,164,56,86,17,21,173,1,183,71,163,113,178,91,34
,2,2,30,113,220,77,129,119,218,40,95,224,157,82,227,251,78,100,41,187,84,174,192,59,163,220,2,111,118,131,110,129,55,131,83,224,205,169,115,244,171,201,253,75,253,47
,11,188,5,218,88,61,22,68,133,214,220,139,103,32,85,124,186,163,66,10,174,210,78,129,55,71,25,47,170,21,60,78,21,124,170,55,21,120,243,200,18,71,115,95,68,150
,196,45,246,100,100,73,224,254,70,150,170,125,193,117,82,102,63,186,36,11,81,88,100,169,10,167,156,58,0,203,57,25,98,123,126,234,194,233,144,215,227,29,189,120,63,228
,181,224,84,70,84,43,55,196,145,165,89,28,89,210,70,53,138,44,77,46,227,200,210,24,142,44,69,131,247,253,141,244,216,206,224,200,82,185,32,178,244,92,74,133,200,82
,64,192,61,227,196,141,37,119,55,92,118,169,7,105,44,205,201,221,112,28,199,51,150,168,175,227,45,62,224,90,52,74,199,91,124,150,252,212,174,58,217,115,127,11,140,165
,74,110,80,83,84,241,96,84,97,163,66,99,105,226,116,24,75,245,30,52,150,186,183,144,177,68,187,225,184,177,52,202,166,188,45,67,166,203,112,55,156,48,150,100,1,27
,137,242,233,222,121,41,234,227,164,172,241,221,3,206,25,211,187,74,70,130,153,161,67,221,251,119,195,137,238,229,110,184,92,36,152,41,152,33,222,13,55,139,119,195,105,99
,9,237,134,227,198,146,187,27,110,12,239,134,139,6,120,55,220,25,188,27,174,92,176,27,238,51,233,67,6,99,41,32,224,158,241,208,213,44,133,2,239,12,119,40,240,94
,245,83,247,165,192,27,215,44,21,20,120,195,205,69,29,39,113,198,208,56,4,151,138,238,239,209,1,238,109,244,32,53,128,251,152,102,160,125,76,71,7,128,210,185,75,80
,12,239,84,22,125,197,75,153,148,225,195,90,206,100,246,184,181,188,212,241,202,153,232,185,80,208,105,11,75,232,4,142,146,106,69,6,125,130,121,188,227,183,130,142,247,210
,88,198,219,223,234,248,160,164,232,175,48,21,16,16,112,183,8,231,44,133,115,150,82,230,39,122,206,18,42,240,118,34,75,242,156,165,7,87,224,237,68,150,156,115,150,58
,41,179,239,158,179,4,10,188,219,225,156,165,112,206,82,134,112,206,82,64,192,41,199,137,27,75,110,205,82,229,39,98,44,57,53,75,243,133,198,82,234,210,183,21,52,150
,100,13,200,156,250,241,141,165,204,29,23,125,200,26,144,14,239,131,6,112,53,165,68,31,67,197,221,113,209,135,172,1,17,125,232,103,44,50,150,50,75,130,247,81,96,44
,57,53,75,220,88,114,106,150,248,110,56,167,102,233,199,53,150,154,167,195,88,10,187,225,194,110,184,176,27,46,32,32,224,209,1,105,78,16,241,110,43,184,67,135,86,67
,112,232,12,25,75,32,201,215,85,48,201,71,134,14,88,30,7,112,121,52,134,14,72,242,145,38,191,226,167,2,2,2,238,140,113,108,111,145,69,8,42,248,38,176,41,70
,22,33,72,215,81,140,12,164,235,200,34,4,167,85,146,69,8,50,121,164,215,64,186,174,64,175,5,4,4,4,0,148,177,143,92,193,142,112,13,59,194,198,85,247,151,243
,140,97,31,185,113,63,20,51,44,2,10,8,8,184,23,124,234,197,231,159,127,206,79,13,177,35,117,135,83,16,64,153,154,130,142,148,217,242,183,231,165,238,225,244,206,190
,151,122,144,167,180,147,14,69,103,180,190,2,11,51,163,202,232,219,128,137,162,159,197,181,141,1,1,1,119,3,58,192,6,212,57,95,130,214,143,209,45,254,183,176,3,77
,28,99,77,249,237,24,138,74,249,125,79,155,189,185,238,163,54,109,148,221,195,52,112,144,125,139,24,239,103,67,140,122,243,31,25,109,242,85,94,37,54,1,27,149,134,230
,153,124,141,108,134,195,167,181,109,134,195,235,105,219,70,125,15,99,147,21,94,31,124,254,198,141,183,222,250,247,158,143,42,66,105,106,234,241,167,188,211,219,193,35,107,166
,215,205,244,197,232,226,177,181,115,229,63,75,122,11,14,84,124,72,148,63,38,97,70,222,191,190,77,27,202,27,148,136,179,75,94,255,193,124,145,214,31,176,136,83,69,222
,87,200,38,51,189,86,130,77,22,249,227,28,244,222,1,179,163,10,59,180,115,3,204,142,58,52,59,204,220,236,1,42,32,32,32,32,224,35,10,113,10,163,180,178,120,249
,142,92,214,101,113,132,88,214,101,113,68,174,72,73,32,91,49,151,29,134,153,50,53,135,234,177,59,236,10,70,172,233,227,176,209,105,193,37,126,131,194,180,172,202,50,18
,110,253,56,3,85,48,78,236,153,207,187,84,150,227,173,187,212,82,214,227,166,100,222,101,183,33,14,232,84,71,172,145,35,80,188,81,64,64,64,64,192,71,28,231,96,210
,223,20,4,172,250,41,42,8,0,145,246,182,2,97,158,56,28,241,142,151,50,33,37,127,52,162,109,22,39,111,52,194,174,163,254,176,66,215,112,222,176,130,13,98,248,11
,181,108,177,179,55,62,96,163,71,222,199,46,237,42,248,216,54,172,224,13,194,216,176,130,63,222,190,156,179,39,82,212,93,123,130,97,211,177,39,24,198,111,16,85,240,213
,209,187,197,244,141,27,55,70,222,104,159,53,58,189,129,187,69,56,157,214,148,242,198,150,140,228,120,195,105,5,99,97,135,194,23,33,178,6,167,119,36,6,57,35,41,129
,13,70,121,159,118,7,62,173,53,177,191,14,111,207,31,73,27,160,167,181,27,11,64,1,15,122,90,155,106,7,243,94,213,79,11,2,202,1,1,1,1,1,1,1,1,1
,1,1,15,10,255,15,121,209,75,222
};



ft_int16_t qsin(ft_uint16_t a)
{
  ft_uint8_t f;
  ft_int16_t s0,s1;

  if (a & 32768)
    return -qsin(a & 32767);
  if (a & 16384)
    a = 32768 - a;
  f = a & 127;
  s0 = ft_pgm_read_word(sintab + (a >> 7));
  s1 = ft_pgm_read_word(sintab + (a >> 7) + 1);
  return (s0 + ((ft_int32_t)f * (s1 - s0) >> 7));
}

/* cos funtion */
ft_int16_t qcos(ft_uint16_t a)
{
  return (qsin(a + 16384));
}
#endif



/*****************************************************************************/
/* Example code to display few points at various offsets with various colors */



/* Boot up for FT800 followed by graphics primitive sample cases */
/* Initial boot up DL - make the back ground green color */
const ft_uint8_t FT_DLCODE_BOOTUP[12] = 
{
  255,255,255,2,//GPU instruction CLEAR_COLOR_RGB
  7,0,0,38, //GPU instruction CLEAR
  0,0,0,0,  //GPU instruction DISPLAY
};

static ft_int32_t ox;
static ft_uint8_t noofch=0;
static ft_uint8_t home_star_icon[] = {0x78,0x9C,0xE5,0x94,0xBF,0x4E,0xC2,0x40,0x1C,0xC7,0x7F,0x2D,0x04,0x8B,0x20,0x45,0x76,0x14,0x67,0xA3,0xF1,0x0D,0x64,0x75,0xD2,0xD5,0x09,0x27,0x17,0x13,0xE1,0x0D,0xE4,0x0D,0x78,0x04,0x98,0x5D,0x30,0x26,0x0E,0x4A,0xA2,0x3E,0x82,0x0E,0x8E,0x82,0xC1,0x38,0x62,0x51,0x0C,0x0A,0x42,0x7F,0xDE,0xB5,0x77,0xB4,0x77,0x17,0x28,0x21,0x26,0x46,0xFD,0x26,0xCD,0xE5,0xD3,0x7C,0xFB,0xBB,0xFB,0xFD,0xB9,0x02,0xCC,0xA4,0xE8,0x99,0x80,0x61,0xC4,0x8A,0x9F,0xCB,0x6F,0x31,0x3B,0xE3,0x61,0x7A,0x98,0x84,0x7C,0x37,0xF6,0xFC,0xC8,0xDD,0x45,0x00,0xDD,0xBA,0xC4,0x77,0xE6,0xEE,0x40,0xEC,0x0E,0xE6,0x91,0xF1,0xD2,0x00,0x42,0x34,0x5E,0xCE,0xE5,0x08,0x16,0xA0,0x84,0x68,0x67,0xB4,0x86,0xC3,0xD5,0x26,0x2C,0x20,0x51,0x17,0xA2,0xB8,0x03,0xB0,0xFE,0x49,0xDD,0x54,0x15,0xD8,0xEE,0x73,0x37,0x95,0x9D,0xD4,0x1A,0xB7,0xA5,0x26,0xC4,0x91,0xA9,0x0B,0x06,0xEE,0x72,0xB7,0xFB,0xC5,0x16,0x80,0xE9,0xF1,0x07,0x8D,0x3F,0x15,0x5F,0x1C,0x0B,0xFC,0x0A,0x90,0xF0,0xF3,0x09,0xA9,0x90,0xC4,0xC6,0x37,0xB0,0x93,0xBF,0xE1,0x71,0xDB,0xA9,0xD7,0x41,0xAD,0x46,0xEA,0x19,0xA9,0xD5,0xCE,0x93,0xB3,0x35,0x73,0x0A,0x69,0x59,0x91,0xC3,0x0F,0x22,0x1B,0x1D,0x91,0x13,0x3D,0x91,0x73,0x43,0xF1,0x6C,0x55,0xDA,0x3A,0x4F,0xBA,0x25,0xCE,0x4F,0x04,0xF1,0xC5,0xCF,0x71,0xDA,0x3C,0xD7,0xB9,0xB2,0x48,0xB4,0x89,0x38,0x20,0x4B,0x2A,0x95,0x0C,0xD5,0xEF,0x5B,0xAD,0x96,0x45,0x8A,0x41,0x96,0x7A,0x1F,0x60,0x0D,0x7D,0x22,0x75,0x82,0x2B,0x0F,0xFB,0xCE,0x51,0x3D,0x2E,0x3A,0x21,0xF3,0x1C,0xD9,0x38,0x86,0x2C,0xC6,0x05,0xB6,0x7B,0x9A,0x8F,0x0F,0x97,0x1B,0x72,0x6F,0x1C,0xEB,0xAE,0xFF,0xDA,0x97,0x0D,0xBA,0x43,0x32,0xCA,0x66,0x34,0x3D,0x54,0xCB,0x24,0x9B,0x43,0xF2,0x70,0x3E,0x42,0xBB,0xA0,0x95,0x11,0x37,0x46,0xE1,0x4F,0x49,0xC5,0x1B,0xFC,0x3C,0x3A,0x3E,0xD1,0x65,0x0E,0x6F,0x58,0xF8,0x9E,0x5B,0xDB,0x55,0xB6,0x41,0x34,0xCB,0xBE,0xDB,0x87,0x5F,0xA9,0xD1,0x85,0x6B,0xB3,0x17,0x9C,0x61,0x0C,0x9B,0xA2,0x5D,0x61,0x10,0xED,0x2A,0x9B,0xA2,0x5D,0x61,0x10,0xED,0x2A,0x9B,0xA2,0x5D,0x61,0x10,0xED,0x2A,0x9B,0xED,0xC9,0xFC,0xDF,0x14,0x54,0x8F,0x80,0x7A,0x06,0xF5,0x23,0xA0,0x9F,0x41,0xF3,0x10,0x30,0x4F,0x41,0xF3,0x18,0x30,0xCF,0xCA,0xFC,0xFF,0x35,0xC9,0x79,0xC9,0x89,0xFA,0x33,0xD7,0x1D,0xF6,0x5E,0x84,0x5C,0x56,0x6E,0xA7,0xDA,0x1E,0xF9,0xFA,0xAB,0xF5,0x97,0xFF,0x2F,0xED,0x89,0x7E,0x29,0x9E,0xB4,0x9F,0x74,0x1E,0x69,0xDA,0xA4,0x9F,0x81,0x94,0xEF,0x4F,0xF6,0xF9,0x0B,0xF4,0x65,0x51,0x08};






static void polarxy(ft_int32_t r, ft_uint16_t th, ft_int32_t *x, ft_int32_t *y)
{
  *x = (16 * (FT_DispWidth/(2*noofch)) + (((long)r * qsin(th)) >> 11) + 16 * ox);
  *y = (16 * 300 - (((long)r * qcos(th)) >> 11));
}


void vertex(ft_int32_t x, ft_int32_t y)
{
  Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(x,y));
}


static void polar(ft_int32_t r, ft_uint16_t th)
{
  ft_int32_t x, y;
  polarxy(r, th, &x, &y);
  vertex(x, y);
}

ft_uint16_t da(ft_int32_t i)
{
  return (i - 45) * 32768L / 360;
}

static void cs(ft_uint8_t i)
{
  switch (i)
  {
    case  0:  Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(200,255,200)); break;
    case 60: Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,0)); break;
    case 80: Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,0,0)); break;
  }
}

ft_void_t home_setup()
{
  Ft_Gpu_Hal_WrCmd32(phost,CMD_INFLATE);
  Ft_Gpu_Hal_WrCmd32(phost,250*1024L);
  Ft_Gpu_Hal_WrCmdBuf(phost,home_star_icon,sizeof(home_star_icon));
  
  Ft_Gpu_CoCmd_Dlstart(phost);        // start
  Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
  Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255, 255, 255));
  Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(13));    // handle for background stars
  Ft_App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(250*1024L));      // Starting address in gram
  Ft_App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT(L4, 16, 32));  // format 
  Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE(NEAREST, REPEAT, REPEAT, 512, 512  ));
  Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(14));    // handle for background stars
  Ft_App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(250*1024L));      // Starting address in gram
  Ft_App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT(L4, 16, 32));  // format 
  Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE(NEAREST, BORDER, BORDER, 32, 32  ));
  Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
  Ft_Gpu_CoCmd_Swap(phost);
  Ft_App_Flush_Co_Buffer(phost);
  Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
}

static ft_uint8_t sk=0;
ft_uint8_t Read_Keys()
{
  static ft_uint8_t Read_tag=0,temp_tag=0,ret_tag=0;	
  Read_tag = Ft_Gpu_Hal_Rd8(phost,REG_TOUCH_TAG);
  ret_tag = NULL;
  if(Read_tag!=NULL)								// Allow if the Key is released
  {
    if(temp_tag!=Read_tag)
    {
      temp_tag = Read_tag;	
      sk = Read_tag;										// Load the Read tag to temp variable	
    }  
  }
  else
  {
    if(temp_tag!=0)
    {
      ret_tag = temp_tag;
    }  
    sk = 0;
  }
  return ret_tag;
}

char *info[] = {  "FT800 Gauges Application",

                          "APP to demonstrate interactive Gauges,",
                          "using Lines & custom Font."
                       };

ft_void_t Info()
{
  ft_uint16_t dloffset = 0,z;
  Ft_CmdBuffer_Index = 0;
  

  
  Ft_Gpu_CoCmd_Dlstart(phost); 
  Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
  Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
  Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,FT_DispHeight/2,26,OPT_CENTERX|OPT_CENTERY,"Please tap on a dot");
  Ft_Gpu_CoCmd_Calibrate(phost,0);
  Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
  Ft_Gpu_CoCmd_Swap(phost);
  Ft_App_Flush_Co_Buffer(phost);
  Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
  
  Ft_Gpu_CoCmd_Logo(phost);
  Ft_App_Flush_Co_Buffer(phost);
  Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
  while(0!=Ft_Gpu_Hal_Rd16(phost,REG_CMD_READ)); 
  dloffset = Ft_Gpu_Hal_Rd16(phost,REG_CMD_DL);
  dloffset -=4;
  Ft_Gpu_Hal_WrCmd32(phost,CMD_MEMCPY);
  Ft_Gpu_Hal_WrCmd32(phost,100000L);
  Ft_Gpu_Hal_WrCmd32(phost,RAM_DL);
  Ft_Gpu_Hal_WrCmd32(phost,dloffset);
  do
  {
    Ft_Gpu_CoCmd_Dlstart(phost);   
    Ft_Gpu_CoCmd_Append(phost,100000L,dloffset);
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(256));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(256));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_B(0));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_C(0));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_D(0));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_E(256));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_F(0));  
    Ft_App_WrCoCmd_Buffer(phost,SAVE_CONTEXT());	
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(219,180,150));
    Ft_App_WrCoCmd_Buffer(phost,COLOR_A(220));
    Ft_App_WrCoCmd_Buffer(phost,BEGIN(EDGE_STRIP_A));
    Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(0,FT_DispHeight*16));
    Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(FT_DispWidth*16,FT_DispHeight*16));
    Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
    Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());	
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
   // INFORMATION 
    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,20,28,OPT_CENTERX|OPT_CENTERY,info[0]);
    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,60,26,OPT_CENTERX|OPT_CENTERY,info[1]);
    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,90,26,OPT_CENTERX|OPT_CENTERY,info[2]);  
//    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,120,26,OPT_CENTERX|OPT_CENTERY,info[3]);  
    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,FT_DispHeight-30,26,OPT_CENTERX|OPT_CENTERY,"Click to play");
    if(sk!='P')
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
    else
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(100,100,100));
    Ft_App_WrCoCmd_Buffer(phost,BEGIN(FTPOINTS));   
    Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(20*16));
    Ft_App_WrCoCmd_Buffer(phost,TAG('P'));
    Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((FT_DispWidth/2)*16,(FT_DispHeight-60)*16));
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(180,35,35));
    Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
    Ft_App_WrCoCmd_Buffer(phost,VERTEX2II((FT_DispWidth/2)-14,(FT_DispHeight-75),14,4));
    Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
    Ft_Gpu_CoCmd_Swap(phost);
    Ft_App_Flush_Co_Buffer(phost);
    Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
  }while(Read_Keys()!='P');
}



void Gauges()
{
  ft_uint16_t w,h,n, val=0,a,th,size_d,temp[1024];
  ft_int32_t rval,tgt;
  ft_uint16_t dloffset;
  ft_uint8_t i,bi,z;
  ft_int32_t x,y,tx,ty,o,dt;
  w = 220; h = 130;
  noofch = 1;
  dt = 50;
  #ifdef SAMAPP_DISPLAY_WQVGA
  noofch = 2;
  dt = 10;
  #endif
 
  Ft_Gpu_CoCmd_Dlstart(phost);   
  Ft_Gpu_CoCmd_MemSet(phost,0,0,10*1024);	
  Ft_App_Flush_Co_Buffer(phost);
  Ft_Gpu_Hal_WaitCmdfifo_empty(phost);

  Ft_Gpu_Hal_WrCmd32(phost,CMD_INFLATE);
  Ft_Gpu_Hal_WrCmd32(phost,0);
  WRITE2CMD(digits);

 
  Ft_Gpu_CoCmd_Dlstart(phost);   	
  Ft_Gpu_CoCmd_SetFont(phost,13,0);
  Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(13));
  Ft_App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(144 - (32L*(54/2)*87)));
  Ft_App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT(L4, 54/2,87));
  Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE(NEAREST, BORDER, BORDER, 54, 87));
  Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
  Ft_Gpu_CoCmd_Swap(phost);
  Ft_App_Flush_Co_Buffer(phost);
  Ft_Gpu_Hal_WaitCmdfifo_empty(phost);	
	
 Ft_Gpu_CoCmd_Dlstart(phost);   							// dl = 0;
 Ft_App_WrCoCmd_Buffer(phost,CLEAR_COLOR_RGB(55,55,55));
 Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
 Ft_App_WrCoCmd_Buffer(phost,CLEAR_COLOR_RGB(0,0,0));
 
 y = 10;
 for(z=0;z<(FT_DispWidth/w);z++)
 {
   ox = 240*z;  
   Ft_App_WrCoCmd_Buffer(phost,SCISSOR_XY(ox+dt,y));
   Ft_App_WrCoCmd_Buffer(phost,SCISSOR_SIZE(w,h));
   Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));			
   Ft_App_WrCoCmd_Buffer(phost,BEGIN(LINES));
   Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(10));
   for (bi = 0; bi < 81; bi += 10) 
   {
     cs(bi);
     for ( i = 2; i < 10; i += 2) 
     {
       a = da(bi + i);
       polar(220, a);
       polar(240, a);
     }
   }
    
   Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(16));
   for (i = 0; i < 91; i += 10)
   {
     cs(i);
     a = da(i);
     polar(220, a);
     polar(250, a);
   }
   Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
   for (i = 0; i < 91; i += 10)
   {
     a = da(i);
     polarxy(260, a, &tx, &ty);
     Ft_Gpu_CoCmd_Number(phost,tx >> 4, ty >> 4,26,OPT_CENTER, i);
   }
   ox = (FT_DispWidth/(2*noofch))+(z*(FT_DispWidth/2));
   if(z==1)Ft_Gpu_CoCmd_Text(phost,ox,h-10,28,OPT_CENTERX,"Random"); 
   if(z==0)Ft_Gpu_CoCmd_Text(phost,ox,h-10,28,OPT_CENTERX,"Resistance");
 }
 Ft_App_Flush_Co_Buffer(phost);
 Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
  
 
 dloffset =  Ft_Gpu_Hal_Rd16(phost,REG_CMD_DL);
 
 
 Ft_Gpu_Hal_WrCmd32(phost,CMD_MEMCPY);
 Ft_Gpu_Hal_WrCmd32(phost,100000L);
 Ft_Gpu_Hal_WrCmd32(phost,RAM_DL);
 Ft_Gpu_Hal_WrCmd32(phost,dloffset);
 y = 10+120+20;
 rval=0,tgt = 4500;
 do
 {
   Ft_Gpu_CoCmd_Dlstart(phost);   
   Ft_Gpu_CoCmd_Append(phost,100000L,dloffset);	
   for(z=0;z<(FT_DispWidth/w);z++)
   {
     ox = (FT_DispWidth/2)*z;
     if(z==0)
     {
	val = Ft_Gpu_Hal_Rd16(phost,REG_TOUCH_RZ);			
	val = 10*min(899,val);			
     }else   if(z==1)
     {
	int d = (tgt - rval) / 16;
	rval += d;
	if (ft_random(60) == 0)
	tgt = ft_random(9000L);
	val = rval;
     }
    Ft_App_WrCoCmd_Buffer(phost,SCISSOR_XY(ox+dt,10));
    Ft_App_WrCoCmd_Buffer(phost,SCISSOR_SIZE(w,120));
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
    Ft_App_WrCoCmd_Buffer(phost,BEGIN(LINES));
    Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(10));
    th = (val - 4500L) * 32768L / 36000L;
    for (o = -5; o < 6; o++) 
    {
      polar(170, th + (o << 5));
      polar(235, th);
    }
    Ft_App_WrCoCmd_Buffer(phost,SCISSOR_XY(ox+dt,y));
    Ft_App_WrCoCmd_Buffer(phost,SCISSOR_SIZE(w,(ft_uint16_t)(FT_DispHeight*0.36)));
    Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,0,0));
    Ft_Gpu_CoCmd_Number(phost,ox+dt+10,160,13,2,val/100);	
    Ft_Gpu_CoCmd_Text(phost,ox+dt+96,160,13,0,".");
    Ft_Gpu_CoCmd_Number(phost,ox+dt+106,160,13,2,val%100);
   }
   Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
   Ft_Gpu_CoCmd_Swap(phost);
   Ft_App_Flush_Co_Buffer(phost);
   Ft_Gpu_Hal_WaitCmdfifo_empty(phost);	
 }
 while(1);
}





#ifdef MSVC_PLATFORM
/* Main entry point */
ft_int32_t main(ft_int32_t argc,ft_char8_t *argv[])
#endif
#ifdef ARDUINO_PLATFORM
ft_void_t setup()
#endif
{
	Ft_Gpu_HalInit_t halinit;
	
	halinit.TotalChannelNum = 1;

              
	Ft_Gpu_Hal_Init(&halinit);
	host.hal_config.channel_no = 0;
#ifdef MSVC_PLATFORM_SPI
	host.hal_config.spi_clockrate_khz = 12000; //in KHz
#endif
#ifdef ARDUINO_PLATFORM_SPI
	host.hal_config.spi_clockrate_khz = 4000; //in KHz
#endif
        Ft_Gpu_Hal_Open(&host);
            
	phost = &host;

    Ft_BootupConfig();

	printf("reg_touch_rz =0x%x ", Ft_Gpu_Hal_Rd16(phost, REG_TOUCH_RZ));
	printf("reg_touch_rzthresh =0x%x ", Ft_Gpu_Hal_Rd32(phost, REG_TOUCH_RZTHRESH));
	printf("reg_touch_tag_xy=0x%x",Ft_Gpu_Hal_Rd32(phost, REG_TOUCH_TAG_XY));
	printf("reg_touch_tag=0x%x",Ft_Gpu_Hal_Rd32(phost, REG_TOUCH_TAG));


    /*It is optional to clear the screen here*/	
    Ft_Gpu_Hal_WrMem(phost, RAM_DL,(ft_uint8_t *)FT_DLCODE_BOOTUP,sizeof(FT_DLCODE_BOOTUP));
    Ft_Gpu_Hal_Wr8(phost, REG_DLSWAP,DLSWAP_FRAME);
    
    Ft_Gpu_Hal_Sleep(1000);//Show the booting up screen. 

	home_setup();
	Info();
	Gauges();   
	/* Close all the opened handles */
    Ft_Gpu_Hal_Close(phost);
    Ft_Gpu_Hal_DeInit();
#ifdef MSVC_PLATFORM
	return 0;
#endif
}

void loop()
{
}



/* Nothing beyond this */













